/* 
Electronic Technologies for Art II
et4a.roberttwomey.com | rtwomey@ucsd.edu

mqtt-stored-credentials

Connects to your local wifi or the UCSD-PROTECTED using your credentials. 
Saves your credentials to NVM so you only enter them once.
Publishes MQTT messages to a broker on shiftr.io
 - view here https://public.cloud.shiftr.io/
 - look for /smartenv/devboard1

 OTE: Be sure to update the topic to a unique name under /smartenv/devboard2

Adapted from https://www.emqx.com/en/blog/esp32-connects-to-the-free-public-mqtt-broker
And my examples from: https://github.com/roberttwomey/smartenv-code/tree/main/particle
*/

#include <WiFi.h>
#include <PubSubClient.h>
#include <Preferences.h>

const int LED2 = 18;
const int SW1 = 12;

const char *topic = "/smartenv/devboard1/";
int timeLastSent = 0;

// MQTT Broker
const char *mqtt_broker = "public.cloud.shiftr.io";
const char *mqtt_username = "public";
const char *mqtt_password = "public";
const int mqtt_port = 1883;

// WIFI information stored in NVM
Preferences preferences;
WiFiClient espClient;
PubSubClient client(espClient);
int counter = 0;

// Function to save WiFi credentials to NVS
void saveWiFiCredentials(const char * ssid, const char * username, const char * password) {
    preferences.begin("WiFiCreds", false);  // Open storage in write mode
    preferences.putString("wifi_ssid", ssid);
    preferences.putString("wifi_user", username);
    preferences.putString("wifi_pass", password);
    preferences.end();
    Serial.println("WiFi credentials saved.");
}

// Function to load WiFi credentials from NVS
bool loadWiFiCredentials(String &ssid, String &username, String &password) {
    preferences.begin("WiFiCreds", true);  // Open storage in read mode
    ssid = preferences.getString("wifi_ssid", "");
    username = preferences.getString("wifi_user", "");
    password = preferences.getString("wifi_pass", "");
    preferences.end();

    if (ssid.length() > 0 && username.length() > 0 && password.length() > 0) {
        Serial.println("WiFi credentials loaded.");
        return true;
    } else {
        Serial.println("No WiFi credentials found. Please compile and upload store-credentials example.");
        return false;
    }
}

void setup() {

  // NOTE: To store your wifi login info:
  // (1) Uncomment the correct line (UCSD-PROTECTED or home) and add your username and password.
  // (2) Compile and upload, run once. This will store your credentials.
  // (3) Delete your credentials (so your password isn't stored in your code)
  // (4) Comment out and reupload. Now the program will log-in with the stored info.

  // saveWiFiCredentials("UCSD-PROTECTED", "YOUR_USERNAME", "YOUR_PASSWORD"); // enterprise wifi (SSID, USERNAME, PASSWORD)
  // saveWiFiCredentials("housemachine", "none", "YOUR_PASSWORD"); // home wifi (SSID/Password)

  String ssid, username, password;

  if (loadWiFiCredentials(ssid, username, password)) {

    Serial.begin(115200);
    delay(10);
    Serial.println();
    Serial.print("Connecting to network: ");
    Serial.println(ssid);
    WiFi.disconnect(true);  //disconnect from wifi to set new wifi connection
    WiFi.mode(WIFI_STA);    //init wifi mode

    if(strstr(ssid.c_str(), "UCSD") != NULL) {
      // start UCSD-PROTECTED
      WiFi.begin(ssid, WPA2_AUTH_PEAP, username, username, password); // UCSD-PROTECTED wifi
    } else {
      // start other WIFI
      WiFi.begin(ssid, password); // home wifi
    }
    
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
      counter++;
      if (counter >= 60) {  //after 30 seconds timeout - reset board
        ESP.restart();
      }
    }
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address set: ");
    Serial.println(WiFi.localIP());  //print LAN IP

    // CONNECT TO MQTT BROKER
    //connecting to a mqtt broker
    client.setServer(mqtt_broker, mqtt_port);
    client.setCallback(callback);
    
    while (!client.connected()) {
        String client_id = "esp32-client-";
        client_id += String(WiFi.macAddress());
        Serial.printf("The client %s connects to the public MQTT broker\n", client_id.c_str());
        if (client.connect(client_id.c_str(), mqtt_username, mqtt_password)) {
            Serial.printf("Public broker %s connected\n", mqtt_broker);
        } else {
            Serial.print("failed with state ");
            Serial.print(client.state());
            delay(2000);
        }
    }
    // Publish and subscribe
    client.publish(topic, "Hi, I'm ESP32 ^^");
    client.subscribe(topic);

    // LEDs
    pinMode(LED2, OUTPUT);
    pinMode(SW1, INPUT);
  }
}

void callback(char *topic, byte *payload, unsigned int length) {
    Serial.print("Message arrived in topic: ");
    Serial.println(topic);
    Serial.print("Message:");
    for (int i = 0; i < length; i++) {
        Serial.print((char) payload[i]);
    }

    // this is where we read the message and respond
    if (strstr((char *)payload, "on") != NULL) {
      digitalWrite(LED2, 1);
    } else if (strstr((char *)payload, "off") != NULL) {
      digitalWrite(LED2, 0);
    }

    // TODO add more

    Serial.println();
    Serial.println("-----------------------");
}

void loop() {
    client.loop();

    // publish a message with the status of SW1
    if(millis()-timeLastSent > 100) {
      if (digitalRead(SW1)) {
        client.publish("/smartenv/devboard1/sw1", "1");
      } else {
        client.publish("/smartenv/devboard1/sw1", "0");
      }
      timeLastSent = millis();
    }
}